let $ = jQuery;

$(document).ready(function () {

    const URL = {
        discipline: {
            map: {
                addModule: g_URLdir + 'handler/map/addModule',
                addSubmodule: g_URLdir + 'handler/map/addSubmodule',
                deleteModule: g_URLdir + 'handler/map/deleteModule',
                swapModules: g_URLdir + 'handler/map/swapModules',
                setModuleName: g_URLdir + 'handler/map/setModuleName',
                setSubModuleMaxAndControl: g_URLdir + 'handler/map/setSubModuleMaxAndControl',
            }
        }
    }, API = URL.discipline.map;

    $.fn.extend({
        sum: function (asText) {
            let sum = 0;
            this.each(function () {
                sum += parseInt(0 + (asText ? $(this).text() : $(this).val()))
            });
            return sum;
        }
    });


    // CSS class must be added to module / submodule
    // DOM-nodes that contains some data-* attributes
    const DATA_CLASS = '_info';

    /** Get the nearest module node and return its data */
    function dataOf(node) {
        return $(node).closest(`.${DATA_CLASS}`).data();
    }


    const templates = {
        $module: $($.parseHTML(
            '<div class="moduleGroup" style="display:none">' +
            '	<div class="moduleHead">' +
            '		<div class="name">' +
            '			<input type="text" class="inputName moduleName" value="" placeholder="Наименование модуля">' +
            '		</div>' +
            '       <div class="currentControl">0</div>' +
            '		<div class="landmarkControl">0</div>' +
            '		<div class="actions">' +
            '			<div class="deleteModule icon delete"></div>' +
            '			<div class="downModule icon down"></div>' +
            '			<div class="upModule icon up"></div>' +
            '		</div>' +
            '	</div>' +
            '	<div class="submodules"></div>' +
            '	<button class="addSubmodule">Добавить мероприятие</button>' +
            '</div>'
        )),

        newModule: (moduleID, submoduleID) => {
            let node = templates.$module.clone().data({id: moduleID, type: 'module'}).addClass(DATA_CLASS);
            node.children('div.submodules').append(templates.newSubmodule(submoduleID));
            return node;
        },

        $submodule: $($.parseHTML(
            '<div class="submodule">' +
            '	<div class="name">' +
            '		<input type="text" class="inputName submoduleName" placeholder="Наименование мероприятия" data-type="submodule">' +
            '	</div>' +
            '	<div class="currentControl"><input type="text" class="inputCredit inputCurrentControl" value="0"></div>' +
            '	<div class="landmarkControl"><input type="text" class="inputCredit inputLandmarkControl" value="0"></div>' +
            '	<div class="actions">' +
            '		<div class="deleteSubmodule icon delete"></div>' +
            '		<div class="moveUp icon up"></div>' +
            '		<div class="moveDown icon down"></div>' +
            '	</div>' +
            '</div>'
        )),

        newSubmodule: (id) => templates.$submodule.clone().data({id: id, type: 'submodule'}).addClass(DATA_CLASS),

        confirm: $($.parseHTML(
            '<div class="confirmDeleteDiv">Уверены? ' +
            '	<span class="confirmDeleteModule">Да</span> ' +
            '	<span class="cancel">Нет</span>' +
            '</div>'
        )),

        submoduleConfirm: $($.parseHTML(
            '<div class="confirmDeleteDiv">Уверены? ' +
            '   <span class="confirmDeleteSubmodule">Да</span> ' +
            '   <span class="cancel">Нет</span>' +
            '</div>'
        ))
    };

    function recountRates($submodules) {
        let maxCredits = $('input.inputCurrentControl', $submodules).sum();
        $submodules.prev().children('div.currentControl').text(maxCredits);

        maxCredits = $('input.inputLandmarkControl', $submodules).sum();
        $submodules.prev().children('div.landmarkControl').text(maxCredits);
    }

    function swapElements($lower, $upper, speed) {
        let lowerRoute = $upper.offset().top - $lower.offset().top,
            upperRoute = lowerRoute + $upper.height() - $lower.height();

        $.when(
            $upper.animate({top: -upperRoute}, speed || 200),
            $lower.animate({top: lowerRoute}, speed || 200)
        ).done(function () {
            $upper.add($lower).css('top', '0px');
            $lower.insertBefore($upper);
            this.animating = false;
        });
    }

    function swapModules($lower, $upper) {
        let lower = dataOf($lower);

        $.postJSON(URL.discipline.map.swapModules, {type: lower.type, ID1: lower.id, ID2: dataOf($upper).id})
            .fail(_ => Popup.error(`Ошибка при изменении порядка ${lower.type === 'module' ? 'модулей' : 'мероприятий'}`))
            .done(_ => swapElements($lower, $upper))
    }


    let $modules = $('.moduleList');

    $modules.on('click', '.moveUp', ({target: node}) => {
        let className = (dataOf(node).type === 'module') ? '.moduleGroup' : '.submodule',
            $current = $(node).parents(className),
            $upper = $current.prev(className);
        console.log('move up');
        $upper.length ? swapModules($current, $upper) : Popup.error('Переместить выше нельзя');
    });

    $modules.on('click', '.moveDown', ({target: node}) => {
        let className = (dataOf(node).type === 'module') ? '.moduleGroup' : '.submodule',
            $current = $(node).parents(className),
            $lower = $current.next(className);
        console.log('move down');
        $lower.length ? swapModules($lower, $current) : Popup.error('Переместить ниже нельзя');
    });


    $('.addModule').click(function () {
        let $addModule = $(this).turnOff();

        $.postJSON(API.addModule, {'id': $addModule.data().discipline})
            .done(({module: mID, submodule: smID}) => {
                let $module = templates.newModule(mID, smID),
                    $place = $('.moduleList').children('.moduleGroup').last();

                $place.length > 0 ? $place.after($module) : $('.moduleList').prepend($module);
                $module.slideDown(200);

                // Remove the stub "No modules"
                $('.empty').remove();
            })
            .fail(_ => Popup.error('Ошибка при добавлении модуля'))
            .always(_ => $addModule.turnOn())
    });

    $modules.on('click', 'button.addSubmodule', ({target: button}) => {
        let $addSubButton = $(button).turnOff(),
            moduleID = dataOf($addSubButton).id;

        $.postJSON(API.addSubmodule, {'module': moduleID})
            .done(({submodule: id}) => {
                let $submodule = templates.newSubmodule(id);
                $addSubButton.siblings('.submodules').append($submodule);
                $submodule.slideDown(200);
            })
            .fail(_ => Popup.error('Ошибка при добавлении мероприятия'))
            .always(_ => $addSubButton.turnOn())
    });


    $modules.on('click', '.deleteModule', ({target: button}) => {
        $(button).parent().hide();
        $(button).parent().parent().append(templates.confirm.clone());
    });

    $modules.on('click', '.deleteSubmodule ', ({target: button}) => {
        $(button).parent().hide();
        $(button).parent().parent().append(templates.submoduleConfirm.clone());
    });


    $modules.on('click', '.confirmDeleteModule', ({target: button}) => {
        let $module = $(button).parents('.moduleGroup'),
            $modules = $('.moduleGroup');

        $.postJSON(API.deleteModule, {id: dataOf($module).id, type: 'module'})
            .done(_ => {
                $module.slideUp(200, _ => $module.remove());

                let $modulesHead = $modules.children('.moduleHead'),
                    currentCtrl = $modulesHead.children('.currentControl').sum(true),
                    landmarkCtrl = $modulesHead.children('.landmarkControl').sum(true);

                let exam = $('div.ExamModule').length ? 40 : 0;
                let isExam = $('div.ExamModule').length > 0;
                let maxPossibleSumRate = isExam ? 60 : 100;
                let $moduleHead = $module.find('.moduleHead'),
                    oldC = parseInt($moduleHead.find('div.currentControl').html()),
                    oldL = parseInt($moduleHead.find('div.landmarkControl').html());

                let currentMaxRate = currentCtrl + landmarkCtrl + exam - oldC - oldL;
                $('span.rateIndicator').html(currentMaxRate);

                if (currentMaxRate === maxPossibleSumRate) {
                    $('#rate_link').attr('href', 'rate');
                    $('#rate_link').removeClass('inactive');

                } else {
                    $('#rate_link').removeAttr('href');
                    $('#rate_link').addClass('inactive');
                }

                // todo: use $.hide() / $.show()
                // todo: move to twig template with style "hidden"
                if ($('.moduleGroup').length === 0)
                    $('.moduleList').prepend('<div class="empty"><p class="notification">Модули отсутствуют</p></div>');
            })
            .fail(_ => Popup.error('Ошибка при удалении модуля'))
    });

    $modules.on('click', '.confirmDeleteSubmodule ', function () {
        let $submodule = $(this).parents('.submodule');

        if ($submodule.siblings('.submodule').andSelf().size() <= 1)
            return Popup.error('Невозможно удалить единственное мероприятие в модуле.');

        recountRates($submodule.parent());

        $.postJSON(API.deleteModule, {id: dataOf($submodule).id, type: 'submodule'})
            .done(_ => {
                let difC = parseInt($submodule.find('div.currentControl').find('input').val()),
                    difL = parseInt($submodule.find('div.landmarkControl').find('input').val());

                let $moduleHead = $submodule.parents('.moduleGroup').find('.moduleHead'),
                    oldC = parseInt($moduleHead.find('div.currentControl').html()),
                    oldL = parseInt($moduleHead.find('div.landmarkControl').html());

                // change module header
                $moduleHead.children('div.currentControl').html(oldC - difC);
                $moduleHead.children('div.landmarkControl').html(oldL - difL);

                // change sum rate
                let rateIndicator = $('span.rateIndicator'),
                    old = +rateIndicator.html();
                let currentMaxRate = old - difC - difL;
                rateIndicator.html(currentMaxRate);


                let exam = $('div.ExamModule').length ? 40 : 0;
                let isExam = $('div.ExamModule').length > 0;
                let maxPossibleSumRate = isExam ? 60 : 100;

                if (currentMaxRate === maxPossibleSumRate) {
                    $('#rate_link').attr('href', 'rate');
                    $('#rate_link').removeClass('inactive');

                } else {
                    $('#rate_link').removeAttr('href');
                    $('#rate_link').addClass('inactive');
                }

                $submodule.slideUp(200, _ => $(this).remove());
            })
            .fail(_ => Popup.error('Ошибка при удалении мероприятия'))
    });

    // Отмена удаления модулей/мероприятий
    $modules.on('click', '.cancel ', ({target: button}) => {
        $(button).parent().parent().children('div.actions').show();
        $(button).parent().remove();
    });


    // Update module & submodule name
    $modules.on('change input paste', 'input.moduleName, input.submoduleName', e => Watcher(500, _ => updateUnitName(e)));

    function updateUnitName(e) {
        let $input = $(e.target),
            newName = $input.val(),
            unit = dataOf($input);

        if ($input.data('last-name') === newName) return;
        $input.data('last-name', $input.val());

        $.postJSON(API.setModuleName, {id: unit.id, type: unit.type, name: newName})
            .fail(_ => Popup.error(`Ошибка при изменении названия ${unit.type === 'module' ? 'модуля' : 'мероприятия'}`))
    }


    // Update rates
    $modules.on('keydown', 'input.inputCredit', keyDownOnlyNumber);

    let oldMaxRate = 0;
    $modules.on('focusin', 'input.inputCurrentControl, input.inputLandmarkControl',
        e => oldMaxRate = $(e.target).select().val());

    $modules.on('focusout', 'input.inputCurrentControl, input.inputLandmarkControl',
        e => setNewRateAndControl($(e.target).turnOff(), oldMaxRate));

    function setNewRateAndControl($rateInput, oldMaxRate) {
        // http://gitlab.mmcs.sfedu.ru/it-lab/grade/issues/315
        if ($rateInput.val() === oldMaxRate) {
            $rateInput.turnOn();
            return;
        }
        let $moduleHeads = $('.moduleHead'),
            currentControl = $moduleHeads.children('.currentControl').sum(true),
            landmarkControl = $moduleHeads.children('.landmarkControl').sum(true);

        // Макс. балл соседней ячейки
        let subtract = parseInt(0 + $rateInput.parent().siblings().children('.inputCredit').val()),
            newMaxRate = parseInt(0 + $rateInput.val()),
            isExam = $('div.ExamModule').length > 0,
            maxPossibleSumRate = isExam ? 60 : 100;

        let rateIndicator = currentControl + landmarkControl + newMaxRate - subtract - oldMaxRate;

        if (rateIndicator > maxPossibleSumRate) {
            if (newMaxRate === 0) {
                $rateInput.val(0);
            } else {
                $rateInput.turnOn().val(oldMaxRate);
                return Popup.error('Сумма баллов превышает максимум');
            }
        }

        if (rateIndicator === maxPossibleSumRate) {
            //let $addModule = $('.addModule');
            //let disciplineID = $addModule.data().discipline;
            //$('#rate_link').attr('href', `discipline/${disciplineID}/rate`);
            $('#rate_link').attr('href', 'rate');
            $('#rate_link').removeClass('inactive');

        } else {
            $('#rate_link').removeAttr('href');
            $('#rate_link').addClass('inactive');
        }

        // Меняем max балл
        if (newMaxRate === oldMaxRate) {
            $rateInput.turnOn();
        } else {
            let control = $rateInput.parent().hasClass('landmarkControl') ? 'landmark' : 'current';

            $.postJSON(API.setSubModuleMaxAndControl, {
                'id': dataOf($rateInput).id,
                'maxRate': newMaxRate,
                'control': control
            }).fail(_ => Popup.error('Ошибка при изменении макс. балла'))
                .always(_ => $rateInput.turnOn());

            // Счетчик баллов
            if (isExam) rateIndicator += 40;
            $('div.rateIndicatorDIV span.rateIndicator').html(rateIndicator);

            if (control === 'current')
                $rateInput.parent().parent().find('div.landmarkControl').children('input.inputLandmarkControl').val(0);
            if (control === 'landmark')
                $rateInput.parent().parent().find('div.currentControl').children('input.inputCurrentControl').val(0);
        }

        recountRates($rateInput.parents('.submodules'));
    }

    $('#copyMapButton').click(function () {
        $.post(g_URLdir + "window/copyStructureDialog", {
            discipline:  g_disciplineID
        },
            initCopyMapDialog);
    });

    let copyMapReady = true;

    function initCopyMapDialog(data) {
        wnd.open(JSON.parse(data));

        if (this.isInit) return;
        this.isInit = true;

        initCopyMapDialog.isInit = false;

        $(document).on('click', '#copyStructureButton', function () {
            const jCopyButton = $('#copyStructureButton');
            if (!copyMapReady)
                return;
            copyMapReady = false;

            $.ajax({
                type: "POST",
                url: g_URLdir + "handler/Discipline/copyMap",
                data: {
                    id: g_disciplineID,
                    disciplineFromID: $('#selectDiscipline').val(),
                },
                success: function (data) {
                    copyMapReady = true;
                    data = $.parseJSON(data);

                    if (data.response.success === true) {
                        EventInspector.success('УКД успешно скопирована!');
                        wnd.hide();
                    } else EventInspector.error('УКД не скопирована');
                },
                error: function () {
                    EventInspector.error('УКД не скопирована');
                },
                complete: function () {
                }
            });
        });
    }

});
